/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package controller;


import util.*;
import agent.*;
import agent.autonomous.*;
import gui.*;
import map.*;
import transport.*;
import transport.protocol.*;

import java.awt.*;
import java.util.*;



/* ====================================================================== */
/** This class is used by the simulation controller to continually send
 *  messages to agents and their corresponding wearable devices.  We can
 *  send time, temperature, soundlevel, location, etc.
 *
 */
public class ControllerThread extends Thread{


	/** The iSIM controller. */
	private Controller controller;




    /* ===================================================================== */
    /** This constructor initializes the thread and starts it running.
     *
     *  @param  controller   The iSIM controller.
     *
     */
	public ControllerThread(Controller controller){

		this.controller = controller;
		this.start();

	} //Constructor




    /* ===================================================================== */
    /** This method is a continuous loop for thread.  Every 5 seconds (may
     *  change)  we send message to all the agents in the simulation.  The time
     *  time interval and the messages to send can be changed.
     *
     */
	public void run(){

		while(true){

			try{ Thread.sleep(5000); }catch(Exception e){}

			Vector users = controller.getUsers();
			User a = null;

			for( int i=0; i <users.size(); i++ ){
				a = (User)users.elementAt(i);

				//check the user's location and adjust vitals
				Point p = a.getLocation();
				Room room = controller.getChartComp().isRoom(p);
				if( room != null && !(a instanceof Fire) && !(a.isDead()) ){
					if( room.getTemperature() > 100 || room.getOxygen() < 1.0 ){
						int oxygen = a.getOxygen();
						oxygen = (int)Math.round( oxygen * (1-((1-room.getOxygen())*.20)) );
						a.setOxygen(oxygen);

						int health = a.getHealth();
						health = (int)Math.round( health * (1-((1-room.getOxygen())*.075)) );
						if( room.getTemperature() > 100 ){
							health = (int)Math.round( health - ((room.getTemperature()-100)*.075) );
						}
						a.setHealth(health);
						//System.out.println( " "+health );

						controller.fireControllerChange();
					}

					//kill if health is zero
					if( a.getHealth() <= 0 ){
						//SocketSender ss = new SocketSender(new DeathMessage(a.getPort(), 2323));
						//ss.start();
						a.setDead(true);
						try{ Thread.sleep(200); }catch(Exception e){}
						controller.fireControllerChange();
					}
				}

				//send out a bio message
				if( !(a instanceof Fire) ){
					if( a.isBioSensor() && a.isBioSensor2() ){
						SocketSender s1 = new SocketSender( new BioMessage(a.getPort(),2323,a.getHealth(), a.getOxygen(), a.getWeight() ));
						s1.start();
					}else if( !a.isBioSensor() ){
						SocketSender s1 = new SocketSender( new ErrorMessage( a.getPort(),2323,new BioMessage(a.getPort(),2323,0, 0, 0 )));
						s1.start();
					}

					if( a.isLocationSensor() && a.isLocationSensor2() ){
						Position pos = a.getPosition();
						Message msg = new NMEAMessage(a.getPort(), 2323, pos.getLatitude(), pos.getLongitude(), pos.getHeading(), pos.getTime());

						SocketSender ss = new SocketSender(msg);
						ss.start();
					}else if( !a.isLocationSensor() ){
						SocketSender s1 = new SocketSender( new ErrorMessage( a.getPort(),2323,new NMEAMessage(a.getPort(), 2323, 0, 0, 0, 0)));
						s1.start();
					}
				}

				//send out a room message.
				//user is in a room, find room, send it
				if( room != null && !(a instanceof Fire) ){
					if( a.isRoomSensor() && a.isRoomSensor2() ){
						SocketSender s2 = new SocketSender( new RoomMessage(a.getPort(),2323, room ));
						s2.start();
					}else if( !a.isRoomSensor() ){
						SocketSender s1 = new SocketSender( new ErrorMessage( a.getPort(),2323,new RoomMessage(a.getPort(),2323, null )));
						s1.start();
					}
				}


				//SocketSender ss = new SocketSender( create and send a message in here );
				//ss.start();


			}

		} //while


    } //method: run



} //class: ControllerThread