/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui;

import map.*;
import util.*;

import java.util.*;
import java.awt.image.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;



/* ====================================================================== */
/** This is the JDialog is designed to be a simple display Window for
 *  display a User's (not access point) information.  Some of the information
 *  can be edited at this point.  Also a photo, of the user's real world
 *  location is displayed if exists.
 *
 */
public class ObjectDialog extends JDialog implements DocumentListener, ChangeListener{

	/** Layout used in this panel. */
    private BorderLayout bLayMain = new BorderLayout();

    /** Layout used in this panel. */
    private GridBagLayout gbagLayInfo = new GridBagLayout();

	/** Main panel to hold all components */
    private JPanel pnlMain = new JPanel();

    /** The parent Frame that owns this JDialog. */
    private Frame parentFrame;


	/** An error notification if user types letters in the duration column. */
	private JDialog errorDialog;

	/** Lots of fields for User's stuff. */
	private JTextField p2Field = new JTextField(14);
	private JTextField p1Field = new JTextField(14);
	private JTextField textField = new JTextField(14);
	private JTextField heightField = new JTextField(14);
	private JTextField bottomField = new JTextField(14);

	private JCheckBox isopen = new JCheckBox(" Open.");

	private JSlider transSlider = new JSlider(0,100);
	private JSlider rfSlider = new JSlider(0,100);
	private JSlider soundSlider = new JSlider(0,100);

	/** A button to close this dialog. */
	private JButton btnClose = new JButton("Close");

	/** The image of the user's location if exists. */
	private ImageIcon icoLogo = new ImageIcon("files/textures/nopic.gif");

	/** A label for the user's location photo. */
	private JLabel lblPicture = new JLabel(icoLogo);

	private Wall wall;
	private Furnishing furnishing;
	private Opening opening;

    /* ===================================================================== */
    /** Constructor that creates this JDialog. The parent and controller must
     *  not be null for this Dialog to function properly.
     *
     *  @param  parent   The owning frame of this dialog.
     *  @param  controller   The simulation controller.
     *
     */
    public ObjectDialog(Object item, Frame parent ) {

		super(parent);

		if( item instanceof Wall ){
			wall = (Wall)item;
        	setTitle("Wall Properties");
		}else if( item instanceof Opening ){
			opening = (Opening)item;
        	setTitle("Opening Properties");
		}else if( item instanceof Furnishing ){
			furnishing = (Furnishing)item;
        	setTitle("Furnishing Properties");
		}

        this.parentFrame = parent;

		// Setup the gui
        try {
          init();
        }
        catch(Exception e) {
          e.printStackTrace();
        }

        // Set the size and location

		this.setSize(620, 380);
		if( item instanceof Wall ){
        	this.setSize(620, 360);
		}else if( item instanceof Opening ){
        	this.setSize(620, 460);
		}else if( item instanceof Furnishing ){
        	this.setSize(620, 420);
		}
		this.pack();


		//Center this dialog on the screen.
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int x = ( screen.width / 2) - (this.getSize().width /2 );
		int y = ( screen.height / 2) - (this.getSize().height /2 );
        this.setLocation(x,y);

		// Show us.
        this.setVisible(true);

    } //Construtor




    // =======================================================================
    /** Initializes the GUI with components.
     *
     */
    private void init() throws Exception {

		// Setup layouts and borders
        bLayMain.setHgap(5);
        bLayMain.setVgap(5);
        pnlMain.setBorder(TTBorder.getBorder("Properties"));
        this.getContentPane().setLayout(bLayMain);
        pnlMain.setLayout(new BorderLayout());

		JPanel btnPanel = new JPanel();
        btnPanel.setLayout(gbagLayInfo);

        btnPanel.add(btnClose, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(5, 5, 5, 5), 0, 0));

		// Setup the action listeners
		btnClose.setMnemonic(KeyEvent.VK_C);
        btnClose.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  btdClose_actionPerformed();
              }
        });



		JLabel lblP1 = new JLabel("Position 1:");
		JLabel lblP2 = new JLabel("Position 2:");
		JLabel lblHeight = new JLabel(" Height:");
		if( opening != null )
			lblHeight.setText("Top:");
		JLabel lblBottom = new JLabel("Bottom:");
		JLabel lblText = new JLabel("Texture filename:");
		JLabel lblisopen = new JLabel("Opening is:");

		JLabel lblTrans = new JLabel("Transparency:");
		JLabel lblRF = new JLabel("RF Degredation:");
		JLabel lblSound = new JLabel("Sound Degredation:");



		soundSlider.setMajorTickSpacing(5);
		soundSlider.setPaintTicks(true);
		soundSlider.setSnapToTicks(true);
		soundSlider.setPaintLabels(true);
		soundSlider.addChangeListener(this);

		Hashtable labelTable = new Hashtable();
		labelTable.put(new Integer(0), new JLabel("0%") );
		labelTable.put(new Integer(100), new JLabel("100%") );
		soundSlider.setLabelTable( labelTable );

		rfSlider.setMajorTickSpacing(5);
		rfSlider.setPaintTicks(true);
		rfSlider.setSnapToTicks(true);
		rfSlider.setPaintLabels(true);
		rfSlider.addChangeListener(this);

		labelTable = new Hashtable();
		labelTable.put(new Integer(0), new JLabel("0%") );
		labelTable.put(new Integer(100), new JLabel("100%") );
		rfSlider.setLabelTable( labelTable );

		transSlider.setMajorTickSpacing(5);
		transSlider.setPaintTicks(true);
		transSlider.setSnapToTicks(true);
		transSlider.setPaintLabels(true);
		transSlider.addChangeListener(this);

		labelTable = new Hashtable();
		labelTable.put(new Integer(0), new JLabel("0%") );
		labelTable.put(new Integer(100), new JLabel("100%") );
		transSlider.setLabelTable( labelTable );


		p1Field.setEditable(false);
		p2Field.setEditable(false);

		// Add stuff to panels
		JPanel stats = new JPanel();
		stats.setLayout(gbagLayInfo);

        pnlMain.add(stats, BorderLayout.EAST);
        pnlMain.add(lblPicture, BorderLayout.WEST);
        this.getContentPane().add(pnlMain, BorderLayout.CENTER);
        this.getContentPane().add(btnPanel, BorderLayout.SOUTH);


        stats.add(lblP1,    new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblP2,     new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblRF,     new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblSound,     new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblText,    new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(lblHeight,   new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

		if( opening != null){
        	stats.add(lblBottom,   new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0,
        	            GridBagConstraints.WEST,
        	            GridBagConstraints.HORIZONTAL,
        	            new Insets(0, 10, 10, 0), 0, 0));
	        stats.add(lblTrans,     new GridBagConstraints(0, 7, 1, 1, 0.0, 0.0,
                   	 	GridBagConstraints.WEST,
                    	GridBagConstraints.HORIZONTAL,
                    	new Insets(0, 10, 10, 0), 0, 0));
 	        stats.add(lblisopen,     new GridBagConstraints(0, 8, 1, 1, 0.0, 0.0,
                   	 	GridBagConstraints.WEST,
                    	GridBagConstraints.HORIZONTAL,
                    	new Insets(0, 10, 10, 0), 0, 0));

		}else if( furnishing != null){

	        stats.add(lblTrans,     new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0,
                   	 	GridBagConstraints.WEST,
                    	GridBagConstraints.HORIZONTAL,
                    	new Insets(0, 10, 10, 0), 0, 0));
		}


        stats.add(p1Field,    new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(p2Field,     new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(rfSlider,     new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(soundSlider,     new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(textField,    new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0,
                     GridBagConstraints.WEST,
                     GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

        stats.add(heightField,   new GridBagConstraints(1, 5, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(0, 10, 10, 0), 0, 0));

		if( opening != null){
        	stats.add(bottomField,   new GridBagConstraints(1, 6, 1, 1, 0.0, 0.0,
        	            GridBagConstraints.WEST,
        	            GridBagConstraints.HORIZONTAL,
        	            new Insets(0, 10, 10, 0), 0, 0));
	        stats.add(transSlider,     new GridBagConstraints(1, 7, 1, 1, 0.0, 0.0,
                   	 	GridBagConstraints.WEST,
                    	GridBagConstraints.HORIZONTAL,
                    	new Insets(0, 10, 10, 0), 0, 0));
	        stats.add(isopen,     new GridBagConstraints(1, 8, 1, 1, 0.0, 0.0,
                   	 	GridBagConstraints.WEST,
                    	GridBagConstraints.HORIZONTAL,
                    	new Insets(0, 10, 10, 0), 0, 0));
		}else if( furnishing != null){

	        stats.add(transSlider,     new GridBagConstraints(1, 6, 1, 1, 0.0, 0.0,
                   	 	GridBagConstraints.WEST,
                    	GridBagConstraints.HORIZONTAL,
                    	new Insets(0, 10, 10, 0), 0, 0));
		}



		// Make our size fixed.
        this.setResizable(false);

		refresh();

    } //method: init



    /* ===================================================================== */
    /** This method is from being a ControllerListener, everytime the Controller
     *  notifies us, we refresh the user information in the all fields, and check
     *  for a new photo of the current location and heading.
     *
     */
	public void refresh(){


		if( wall != null ){

			heightField.getDocument().removeDocumentListener(this);
			textField.getDocument().removeDocumentListener(this);
			soundSlider.removeChangeListener(this);
			rfSlider.removeChangeListener(this);

			Position p1 = wall.getPosition1();
			Position p2 = wall.getPosition2();

			p1Field.setText("Lat: "+ (int)(p1.getLatitude()/100) +"."+ (int)(((p1.getLatitude()/100)-(int)(p1.getLatitude()/100))/60*100000000) + ",   Lng: "+(int)(p1.getLongitude()/100) +"."+ (int)(((p1.getLongitude()/100)-(int)(p1.getLongitude()/100))/60*-100000000) );
			p2Field.setText("Lat: "+ (int)(p2.getLatitude()/100) +"."+ (int)(((p2.getLatitude()/100)-(int)(p2.getLatitude()/100))/60*100000000) + ",   Lng: "+(int)(p2.getLongitude()/100) +"."+ (int)(((p2.getLongitude()/100)-(int)(p2.getLongitude()/100))/60*-100000000) );

			textField.setText(wall.getTexture());
			heightField.setText(""+wall.getHeight());

			soundSlider.setValue((int)(wall.getSoundDegredation()*100));
			rfSlider.setValue((int)(wall.getRFDegredation()*100));

			heightField.getDocument().addDocumentListener(this);
			textField.getDocument().addDocumentListener(this);
			soundSlider.addChangeListener(this);
			rfSlider.addChangeListener(this);

		}else if( opening != null ){

			heightField.getDocument().removeDocumentListener(this);
			textField.getDocument().removeDocumentListener(this);
			bottomField.getDocument().removeDocumentListener(this);
			soundSlider.removeChangeListener(this);
			transSlider.removeChangeListener(this);
			rfSlider.removeChangeListener(this);
			isopen.removeChangeListener(this);

			Position p1 = opening.getPosition1();
			Position p2 = opening.getPosition2();

			p1Field.setText("Lat: "+ (int)(p1.getLatitude()/100) +"."+ (int)(((p1.getLatitude()/100)-(int)(p1.getLatitude()/100))/60*100000000) + ",   Lng: "+(int)(p1.getLongitude()/100) +"."+ (int)(((p1.getLongitude()/100)-(int)(p1.getLongitude()/100))/60*-100000000) );
			p2Field.setText("Lat: "+ (int)(p2.getLatitude()/100) +"."+ (int)(((p2.getLatitude()/100)-(int)(p2.getLatitude()/100))/60*100000000) + ",   Lng: "+(int)(p2.getLongitude()/100) +"."+ (int)(((p2.getLongitude()/100)-(int)(p2.getLongitude()/100))/60*-100000000) );

			textField.setText(opening.getTexture());
			heightField.setText(""+opening.getTop());
			bottomField.setText(""+opening.getBottom());

			soundSlider.setValue((int)(opening.getSoundDegredation()*100));
			rfSlider.setValue((int)(opening.getRFDegredation()*100));
			transSlider.setValue((int)(opening.getTransparency()*100));
			isopen.setSelected(opening.isOpen());

			heightField.getDocument().addDocumentListener(this);
			textField.getDocument().addDocumentListener(this);
			bottomField.getDocument().addDocumentListener(this);
			soundSlider.addChangeListener(this);
			rfSlider.addChangeListener(this);
			transSlider.addChangeListener(this);
			isopen.addChangeListener(this);

		}else if( furnishing != null ){

			heightField.getDocument().removeDocumentListener(this);
			textField.getDocument().removeDocumentListener(this);
			soundSlider.removeChangeListener(this);
			transSlider.removeChangeListener(this);
			rfSlider.removeChangeListener(this);

			Position p1 = furnishing.getPosition1();
			Position p2 = furnishing.getPosition2();

			p1Field.setText("Lat: "+ (int)(p1.getLatitude()/100) +"."+ (int)(((p1.getLatitude()/100)-(int)(p1.getLatitude()/100))/60*100000000) + ",   Lng: "+(int)(p1.getLongitude()/100) +"."+ (int)(((p1.getLongitude()/100)-(int)(p1.getLongitude()/100))/60*-100000000) );
			p2Field.setText("Lat: "+ (int)(p2.getLatitude()/100) +"."+ (int)(((p2.getLatitude()/100)-(int)(p2.getLatitude()/100))/60*100000000) + ",   Lng: "+(int)(p2.getLongitude()/100) +"."+ (int)(((p2.getLongitude()/100)-(int)(p2.getLongitude()/100))/60*-100000000) );

			textField.setText(furnishing.getTexture());
			heightField.setText(""+furnishing.getHeight());

			soundSlider.setValue((int)(furnishing.getSoundDegredation()*100));
			rfSlider.setValue((int)(furnishing.getRFDegredation()*100));
			transSlider.setValue((int)(furnishing.getTransparency()*100));

			heightField.getDocument().addDocumentListener(this);
			textField.getDocument().addDocumentListener(this);
			soundSlider.addChangeListener(this);
			rfSlider.addChangeListener(this);
			transSlider.addChangeListener(this);

		}

			try{

				icoLogo = new ImageIcon("files/textures/"+textField.getText());
				if( icoLogo.getIconHeight() < 10 ){

					icoLogo = new ImageIcon("files/textures/nopic.gif");
				}

				lblPicture.setIcon(icoLogo);


			}catch(Exception exc){

				icoLogo = new ImageIcon("files/textures/nopic.gif");
				lblPicture.setIcon(icoLogo);
			}

	} //method: refresh



	public void stateChanged(ChangeEvent e){

		if( wall != null){

			wall.setSoundDegredation( soundSlider.getValue()/100.0 );
			wall.setRFDegredation( rfSlider.getValue()/100.0 );

		}else if( opening != null){

			opening.setSoundDegredation( soundSlider.getValue()/100.0 );
			opening.setRFDegredation( rfSlider.getValue()/100.0 );
			opening.setTransparency( transSlider.getValue()/100.0 );
			opening.setOpen( isopen.isSelected() );

		}else if( furnishing != null){

			furnishing.setSoundDegredation( soundSlider.getValue()/100.0 );
			furnishing.setRFDegredation( rfSlider.getValue()/100.0 );
			furnishing.setTransparency( transSlider.getValue()/100.0 );
		}

	}



   	/* ====================================================================== */
	/** This method is called whenever user edits a value in JTextField.
	 *
	 *  @param  e  The document event from one of our textfields.
	 *
	 */
    public void insertUpdate(DocumentEvent e) {

		//Try to grab text from textfields and save it to the current cd.
   	 	try{

			//set these values.  year must be an int or else it will throw
			// exception.
			if( wall != null){

				wall.setHeight( Integer.parseInt(heightField.getText()) );
				wall.setTexture( textField.getText() );

			}else if( opening != null){

				opening.setTop( Integer.parseInt(heightField.getText()) );
				opening.setBottom( Integer.parseInt(bottomField.getText()) );
				opening.setTexture( textField.getText() );

			}else if( furnishing != null){

				furnishing.setHeight( Integer.parseInt(heightField.getText()) );
				furnishing.setTexture( textField.getText() );
			}

			//controller.removeControllerListener(this);
			//controller.fireControllerChange();
			//controller.addControllerListener(this);

		//This exception is thrown if the uses types a non-numeric character
		// into the duration column.
		}catch(NumberFormatException nfe){

			//This is outdated. Just use java's predefined dialog for this.

			//Setup a current/new JDialog.
			errorDialog = new JDialog(this, "Error");

			//Create a button and the labels.
			JLabel lbl = new JLabel("Please only enter numbers in the height field.");
			JLabel lblPic = new JLabel( new ImageIcon("files/photos/info.jpg") );
			JButton btn = new JButton( "OK");

			//Create panels for hold the Components
			JPanel btnpanel = new JPanel();
			JPanel lblpanel = new JPanel();
			lblpanel.setBorder( BorderFactory.createEmptyBorder(15, 15, 5, 15) );

			//Add components to the panels, then add panels to the JDialog.
			lblpanel.add(lblPic, BorderLayout.WEST);
			lblpanel.add(lbl, BorderLayout.EAST);
			btnpanel.add(btn);

			errorDialog.getContentPane().add(lblpanel, BorderLayout.CENTER);
			errorDialog.getContentPane().add(btnpanel, BorderLayout.SOUTH);

			//Make a button listener to close the dialog.
			btn.setMnemonic(KeyEvent.VK_O);
			btn.addActionListener(new ActionListener() {
            	public void actionPerformed(ActionEvent e) {
            	    errorDialog.dispose();
            	}
        	});

			//Pack it all together
        	errorDialog.pack();

			//Center this dialog on the screen.
			Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
			int x = ( screen.width / 2) - (errorDialog.getSize().width /2 );
			int y = ( screen.height / 2) - (errorDialog.getSize().height /2 );
			errorDialog.setLocation( x, y);

			//Now show this dialog.
        	errorDialog.setVisible(true);


			//Now make sure dialog is in front and in focus.
        	errorDialog.toFront();


		} //try

    } //method: insertUpdate


   	/* ====================================================================== */
	/** Don't care about this method.  Only need it to satisfy DocumentListener
	 *  interface.
	 *
	 *  @param  e  The event we don't care about.
	 *
	 */
    public void removeUpdate(DocumentEvent e){}
    /* ====================================================================== */
 	/** Don't care about this method.  Only need it to satisfy DocumentListener
 	 *  interface.
 	 *
 	 *  @param  e  The event we don't care about.
 	 *
	 */
 	public void changedUpdate(DocumentEvent e){}
    /* ====================================================================== */
 	/** Don't care about this method.  Only need it to satisfy DocumentListener
 	 *  interface.
 	 *
 	 *  @param  e  The event we don't care about.
 	 *
	 */
 	private void calculateValue(DocumentEvent e){}


	public void btdClose_actionPerformed(){

		this.dispose();

	}


} //class: PropDialog
