/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux, adapted from Richard Caudle
 * @version 1.0
 * </pre>
 */

package state;

import gui.*;
import state.events.*;
import transport.*;
import transport.protocol.*;

import java.awt.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.*;
import java.util.*;
import java.awt.event.*;



public class StateClient extends JFrame implements SocketReceiverListener{

	/** Layout used in this panel. */
    private BorderLayout bLayMain = new BorderLayout();

    /** Layout used in this panel. */
    private GridBagLayout gbagLayInfo = new GridBagLayout();

    //Components in this JPanel

    /** Panel for buttons */
    private JPanel btnPanel = new JPanel();

	JLabel msglbl = new JLabel("No received messages");




	int port;


    /** Scrollpane for JTable */
    private JScrollPane spnEvents = new JScrollPane();

    /** JTable for displaying users */
    private JTable tableEvents = new JTable();

    /** The JTable's columns. */
    private String[] colNames = { "Message", "To (port)", "From (port)" };


	private Vector events = new Vector();



	//Constructs a UI which is used by 'widget' and displays info on 'user'
	public StateClient(int port, int i){

		super("State Client "+port);

		this.port = port;

		setLocation(i*50,i*50);


		// Set the borders and layout
        bLayMain.setHgap(5);
        bLayMain.setVgap(5);

        JPanel mainp = new JPanel();

        mainp.setBorder(TTBorder.getBorder("Received Events"));
        mainp.setLayout(new BorderLayout());

        this.getContentPane().setLayout(bLayMain);
        btnPanel.setLayout(gbagLayInfo);

		//  Add stuff to the panels and table
        this.getContentPane().add(btnPanel, BorderLayout.SOUTH);

        btnPanel.add(msglbl, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(5, 10, 10, 0), 0, 0));



		// Set up the table and scrollpane
        mainp.add(spnEvents,  BorderLayout.CENTER);
        this.getContentPane().add(mainp,  BorderLayout.CENTER);

		String[][] rowData = new String[0][colNames.length]; // The data array


        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableEvents = new JTable(dtm);

        spnEvents.getViewport().add(tableEvents, 0);



		// Set panel size (gets over ridden by TrayDialog's set size)
        this.setSize(new Dimension(350, 200));

        refresh();



		SocketReceiver sr = new SocketReceiver(this, port);
		sr.start();

		this.setVisible(true);


	} //Constructor






	public void incomingMessage(Message msg){


		events.add(msg);
		msglbl.setText("Received event check above table.");
		refresh();

	}


	private void refresh(){

		Message msg;

		String[][] rowData = new String[events.size()][colNames.length]; // The data array

		for (int i=0; i < events.size(); i++) {
			msg = (Message)events.get(i);

			if( msg instanceof UniEvent ){
		  		rowData[i][0] = "Unicast message";
		  		rowData[i][1] = ""+((UnicastMessage)msg).getToPeer();
		  		rowData[i][2] = ""+msg.getFrom();

			}else if( msg instanceof MultiEvent ){
		  		rowData[i][0] = "Multicast message";
		  		rowData[i][1] = "n/a";
		  		rowData[i][2] = ""+msg.getFrom();

			}else if( msg instanceof SimEvent ){
		  		rowData[i][0] = "Sim message";
		  		rowData[i][1] = ""+msg.getTo();
		  		rowData[i][2] = "n/a";

			}


		}

        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableEvents = new JTable(dtm);

        spnEvents.getViewport().add(tableEvents, 0);

	}



} //class: