/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package state;

import state.events.*;
import transport.protocol.*;
import gui.*;

import java.awt.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.*;
import java.util.*;
import java.awt.event.*;



/* ====================================================================== */
/** This class is the People tab in the TrayDialog.  It displays a list
 *  of all users in the simulation, and enables adding and removing of users.
 *
 */
public class StatePanel extends JPanel {

	private StateMachine statemachine;

	/** Layout used in this panel. */
    private BorderLayout bLayMain = new BorderLayout();

    /** Layout used in this panel. */
    private GridBagLayout gbagLayInfo = new GridBagLayout();

    //Components in this JPanel

    /** Panel for buttons */
    private JPanel btnPanel = new JPanel();

    /** Increment event button. */
    private JButton btnIncrement = new JButton(" Execute event ");

    /** Decrement event button. */
    private JButton btnDecrement = new JButton(" Rewind event ");


    /** Scrollpane for JTable */
    private JScrollPane spnEvents = new JScrollPane();

    /** JTable for displaying users */
    private JTable tableEvents = new JTable();

    /** The JTable's columns. */
    private String[] colNames = { "Message", "To (port)", "From (port)" };





    // ======================================================================
    /** Constructor sets up this Panel for a tab and calls init.
     *  It also signs us up as a controller listener.
     *
     */
    public StatePanel( StateMachine statemachine ) {

		this.statemachine = statemachine;

        try {
            init();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }

    } //Constructor




    // ======================================================================
    /** Initializes this JPanel with Components.  Adds the table, buttons,
     *  and sets the action listeners.
     *
     */
    private void init() throws Exception {

		// Set the borders and layout
        bLayMain.setHgap(5);
        bLayMain.setVgap(5);

        JPanel mainp = new JPanel();

        mainp.setBorder(TTBorder.getBorder("Event table"));
        mainp.setLayout(new BorderLayout());

        this.setLayout(bLayMain);
        btnPanel.setLayout(gbagLayInfo);

		//  Add stuff to the panels and table
        this.add(btnPanel, BorderLayout.SOUTH);

        btnPanel.add(btnDecrement, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.NONE,
                    new Insets(5, 10, 10, 0), 0, 0));


        btnPanel.add(btnIncrement, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                    GridBagConstraints.WEST,
                    GridBagConstraints.HORIZONTAL,
                    new Insets(5, 10, 10, 0), 0, 0));


		// Set up the table and scrollpane
        mainp.add(spnEvents,  BorderLayout.CENTER);
        this.add(mainp,  BorderLayout.CENTER);

		String[][] rowData = new String[0][colNames.length]; // The data array


        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableEvents = new JTable(dtm);

        spnEvents.getViewport().add(tableEvents, 0);


		// Setup the action listeners
		btnIncrement.setMnemonic(KeyEvent.VK_E);
        btnIncrement.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(ActionEvent e) {
                  btnIncrement_actionPerformed();
              }
        });
		btnDecrement.setMnemonic(KeyEvent.VK_R);
        btnDecrement.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                btnDecrement_actionPerformed();
            }
        });

		// Set panel size (gets over ridden by TrayDialog's set size)
        this.setSize(new Dimension(550, 300));

        setup();
        refresh();

    } //method: init




    // ========================================================================
    /** This method is called to add a new user to the simulation.
     *
     */
    public void btnDecrement_actionPerformed() {

		statemachine.decrementState();
		refresh();


    } //method:




    // ========================================================================
    /** This method is called to remove a user from the simulation.
     *
     */
    public void btnIncrement_actionPerformed() {

		statemachine.incrementState();
		refresh();


    } //method: btdRemove_actionPerformed



	private void setup(){

		Vector events = statemachine.getEvents();
		Message msg;

		String[][] rowData = new String[events.size()][colNames.length]; // The data array

		for (int i=0; i < events.size(); i++) {
			msg = (Message)events.get(i);

			if( msg instanceof UniEvent ){
		  		rowData[i][0] = "Unicast message";
		  		rowData[i][1] = ""+((UnicastMessage)msg).getToPeer();
		  		rowData[i][2] = ""+msg.getFrom();

			}else if( msg instanceof MultiEvent ){
		  		rowData[i][0] = "Multicast message";
		  		rowData[i][1] = "n/a";
		  		rowData[i][2] = ""+msg.getFrom();

			}else if( msg instanceof SimEvent ){
		  		rowData[i][0] = "Sim message";
		  		rowData[i][1] = ""+msg.getTo();
		  		rowData[i][2] = "n/a";

			}


		}

        DefaultTableModel dtm = new DefaultTableModel(rowData, colNames) {
            // set this to non-editable table model.
            public boolean isCellEditable(int row, int col) {
                return false;
            }
        };
        tableEvents = new JTable(dtm);

        spnEvents.getViewport().add(tableEvents, 0);

	}



    // ========================================================================
    /** This method required by ControllerListener interface.
     *  This method resets the data in the JTable, so as to keep it up to date.
     *
     */
	public void refresh(){

        tableEvents.setRowSelectionInterval(statemachine.getCurrentStateIndex(),statemachine.getCurrentStateIndex());


	} //method: refresh


} //class: PnlTabPeople